==> snapstraction-0.3.5a1-py2-none-any.whl: snapstraction/flash.py <==
"""SNAPstraction Serial/Flash ops"""
################################################################
# Copied from flasher.FlashUpgrader, mostly. Adapted for CLI and futures
import array
import binascii
import bz2
import functools
import importlib
import logging
import os
import sys
import tempfile

from distutils.sysconfig import get_python_lib

from flasher.Flasher import Hcs08Flasher
from flasher.ATmega128RFA1Flasher import ATMegaFlasher

from apy import EventScheduler

from tornado.ioloop import IOLoop, PeriodicCallback, TimeoutError
from tornado.concurrent import Future
from tornado.gen import coroutine, Return

import serialwrapper

log = logging.getLogger(__name__)

RF100_MODULES = ['RF100']


class FlasherError(Exception):
    pass


def get_flasher_for_module(module, *args, **kwargs):
    """Returns the flasher class for the module type"""
    if module in RF100_MODULES:
        return Hcs08Flasher(*args, **kwargs)
    else:
        return ATMegaFlasher(*args, **kwargs)


def _get_file_path_for_version(metadata, module, version=None):
    img_dir = os.path.join(metadata._get_meta_dir(), '..', 'images')
    filename = metadata.get_metadata(module, version, aes=True)['filename']
    return os.path.join(img_dir, filename)


def firmware(profile, module, core, sniffer=False, override_file=None, timeout=180, progress_cb=None, finished_cb=None):
    """Flash firmware"""
    firmware = None
    if override_file:
        firmware = os.path.abspath(os.path.expanduser(override_file))
        log.warn("Overriding firmware path: %s", firmware)
    else:
        # Extend the path so we can find firmware support packages
        sys.path.append(get_python_lib())

        # This is a check for "Are we being executed under an active virtualenv?"
        # Different from "Are we installed in a virtualenv"
        if 'VIRTUAL_ENV' in os.environ:
            sys.path.append(os.path.abspath(get_python_lib(prefix=os.environ['VIRTUAL_ENV'])))

        try:
            if sniffer:
                metadata_module_name = "snap_sniffer"
                metadata = importlib.import_module(metadata_module_name + '.metadata')
                log.debug("Looking for sniffer firmware for %s", metadata_module_name)

                firmware = _get_file_path_for_version(metadata, module, core)
            else:
                metadata_module_name = "snap_firmware_%s" % (core.replace('.', '_'),)
                metadata = importlib.import_module(metadata_module_name + '.metadata')
                log.debug("Looking for firmware for %s", metadata_module_name)

                firmware = _get_file_path_for_version(metadata, module, core)

            log.debug("Found firmware %s", firmware)
        except:
            log.exception("Couldn't find firmware module %s, pip install %s", metadata_module_name)
            raise

    # Convert SFI to HEX...
    if os.path.splitext(firmware)[1] == '.sfi':
        fd, path = tempfile.mkstemp(text=True)
        sfi_contents = open(firmware, 'rb').read()
        decompressText = bz2.decompress(sfi_contents)
        if decompressText is not None:
            tmpFile = os.fdopen(fd, 'w')
            tmpFile.write(decompressText)
            tmpFile.close()
            firmware = path

    cb = functools.partial(flasher, profile, module, firmware, "Flash Firmware", progress_cb=progress_cb, finished_cb=finished_cb)
    flash_result = None

    try:
        flash_result = IOLoop.current().run_sync(cb, timeout=timeout)
    except TimeoutError:
        log.error("Flash firmware timed out or aborted!")

    return flash_result


def default_nv(profile, module, timeout=30, progress_cb=None, finished_cb=None):
    """Factory default the NV params"""
    fd, path = tempfile.mkstemp(text=True)
    tempFile = os.fdopen(fd, 'w')

    if module in RF100_MODULES:
        tempFile.write(buildMagicSRec('N'))
    else:
        tempFile.write(buildMagicHRec('N'))

    tempFile.close()

    cb = functools.partial(flasher, profile, module, path, "Default NV Params", progress_cb=progress_cb, finished_cb=finished_cb)
    flash_result = None
    try:
        flash_result = IOLoop.current().run_sync(cb, timeout=timeout)
    except TimeoutError:
        log.error("Default NV Params timed out or aborted!")

    return flash_result


def erase_script(profile, module, timeout=30, progress_cb=None, finished_cb=None):
    """Erase the SNAPpy script"""
    fd, path = tempfile.mkstemp(text=True)
    tempFile = os.fdopen(fd, 'w')

    if module in RF100_MODULES:
        tempFile.write(buildMagicSRec('S'))
    else:
        tempFile.write(buildMagicHRec('S'))

    tempFile.close()

    cb = functools.partial(flasher, profile, module, path, "Erase SNAPpy Script", progress_cb=progress_cb, finished_cb=finished_cb)
    flash_result = None
    try:
        flash_result = IOLoop.current().run_sync(cb, timeout=timeout)
    except TimeoutError:
        log.error("Erase Script timed out or aborted!")

    return flash_result


@coroutine  # noqa
def flasher(profile, module, path, action, progress_cb=None, finished_cb=None):
    """Workhorse, actually runs the flasher"""
    result = Future()

    evScheduler = EventScheduler.EventScheduler()

    serialwrapper_path = os.path.dirname(serialwrapper.__file__)

    def on_finished():
        """Complete callback"""
        log.info("%s complete!", action)
        if finished_cb:
            finished_cb()
        result.set_result(True)

    sc_profile = profile.as_sc_params()

    f = get_flasher_for_module(module,
                               path,
                               scheduler=evScheduler,
                               finishedCallback=on_finished,
                               port=sc_profile.port,
                               type=sc_profile.type,
                               pathToUsbLibrary=serialwrapper_path)

    log.debug("Flasher Instance is: %r", f)
    log.debug("Flasher Vars: %r", vars(f))

    if module not in RF100_MODULES:
        # Force the flasher to idle
        f.state = f.STATE_IDLE

        def go_flasher():
            """Tell the flasher it can wait for incoming data now"""
            f.state = f.STATE_INCOMING_WAIT
    else:
        # RF100 / Hcs08Flasher doesn't have the same state machine
        def go_flasher():
            """Don't do anything"""
            pass

    def on_progress(counter):
        """Progress update callback"""
        log.debug("Got flasher update: %r", counter)
        if progress_cb:
            progress_cb(counter, f.max_progress)

    def on_error(error):
        """Error callback"""
        log.error("Flasher error: %r", error)
        result.set_exception(FlasherError(error))

    f.registerErrorCallback(on_error)
    f.registerProgressCallback(on_progress)

    log.warn("Restart your device. (We'll try to restart it for you if it's an SS200 / reset-on-DTR.)")
    if module in RF100_MODULES:
        log.warn("RF100s currently don't reset-on-DTR, you must reset them manually.")

    if os.path.exists('/etc/e20_version'):
        log.info("Looks like you're on an E20, we'll attempt to reset via GPIO 34")

        def e20_reset_on():
            """Toggle the E20 radio reset pin low"""
            try:
                with open('/sys/class/gpio/gpio34/value', 'w') as gpio:
                    gpio.write('0')
            except:
                log.exception("Couldn't reset SNAP node! (Do you need sudo?)")
                raise

        def e20_reset_off():
            """Toggle the E20 radio reset pin back to high"""
            try:
                with open('/sys/class/gpio/gpio34/value', 'w') as gpio:
                    gpio.write('1')
            except:
                log.exception("Couldn't reset SNAP node! (Do you need sudo?)")
                raise

        evScheduler.schedule(1.001, e20_reset_on)
        evScheduler.schedule(1.200, e20_reset_off)
        evScheduler.schedule(1.1, go_flasher)

    # If it's a SNAPStick, poke the reset line
    elif sc_profile.type == serialwrapper.PyserialDriver.PyserialWrapper.TYPE_SNAPSTICK:
        # Empirically determined schedule timing for reset...
        def bitbang_SS200_reset_on():
            """Bitbang the SS200 reset line low"""
            f.serialDrv.serial.setBitMode(0x10, 0x20)

        def bitbang_SS200_reset_off():
            """Bitbang the SS200 reset line back to high"""
            f.serialDrv.serial.setBitMode(0x11, 0x20)

        evScheduler.schedule(2.001, bitbang_SS200_reset_on)
        evScheduler.schedule(2.200, bitbang_SS200_reset_off)
        # Tell the flasher it can start looking at data after the resets
        evScheduler.schedule(2.100, go_flasher)

    else:
        def try_setDTR():
            """Try to set DTR"""
            try:
                log.debug("setting DTR")
                # Yep, this is inverted on purpose
                f.serialDrv.serial.setDTR(False)
            except:
                log.exception("Couldn't setDTR")

        def try_clearDTR():
            """Try to clear DTR"""
            try:
                log.debug("clearing DTR")
                # Yep, this is inverted on purpose
                f.serialDrv.serial.setDTR(True)
            except:
                log.exception("Couldn't setDTR")

        # Poke the DTR line
        evScheduler.schedule(1.001, try_setDTR)
        evScheduler.schedule(1.200, try_clearDTR)
        # Tell the flasher it can start looking at data after the resets
        evScheduler.schedule(1.100, go_flasher)

    PeriodicCallback(evScheduler.poll, 5).start()

    done = yield result
    raise Return(done)

################################
# These are copied from flasher, because the file they're in now also imports wx...
# Favor duplication over dealing with wxPython...


def s19_checksum(line):
    """Calculate the S19 checksum"""
    length = int(line[2:4], 16)
    pbytes = [int(line[i * 2:i * 2 + 2], 16) for i in range(1, length + 1)]
    return ~sum(pbytes) & 0xff


def niceHex(number):
    """Format a number in a nice hex way?"""
    numHex = hex(number)
    numHex = numHex[2:]  # chop off the leading '0x'
    if len(numHex) != 2:
        numHex = '0' + numHex  # tack on a leading '0' if not two digits

    numHex = numHex.upper()
    return numHex


def buildMagicSRec(cmdChar):
    """Build the magic S19 record used for defaulting NV params and erasing scripts"""
    magicAddr = '1080'
    magicKey = '524D44454D474B42'

    cmdHex = niceHex(ord(cmdChar))

    magicLen = niceHex((len(magicAddr) + len(magicKey) + len(cmdHex) + 2) / 2)  # +2 for trailing checksum, /2 to get bytes not chars

    srec = 'S1' + magicLen + magicAddr + magicKey + cmdHex

    checksum = s19_checksum(srec)
    srec = srec + niceHex(checksum)

    return srec


def buildMagicHRec(cmdChar, addr='02F0'):
    """Build the magic hex record used for defaulting NV params and erasing scripts"""
    magicKey = '524D44454D474B42'
    record = '09%s00%s%02X' % (addr, magicKey, ord(cmdChar))
    crc = (~sum(array.array('B', binascii.unhexlify(record))) + 1) % 2 ** 8

    # text = ':020000020000FC\n' # Header
    text = ':%s%02X\n' % (record, crc)
    text += ':00000001FF'  # End of file record

    return text

==> snapstraction-0.8.1-py2-none-any.whl: snapstraction/flash.py <==
"""SNAPstraction Serial/Flash ops"""
################################################################
# Copied from flasher.FlashUpgrader, mostly. Adapted for CLI and futures
import array
import binascii
import bz2
import functools
import importlib
import logging
import os
import tempfile

from flasher.Flasher import Hcs08Flasher
from flasher.ATmega128RFA1Flasher import ATMegaFlasher

from apy import EventScheduler

from tornado.ioloop import IOLoop, PeriodicCallback, TimeoutError
from tornado.concurrent import Future
from tornado.gen import coroutine, Return

import serialwrapper

from util import ensure_path_contains_python_lib

log = logging.getLogger(__name__)

RF100_MODULES = ['RF100']


class FlasherError(Exception):
    pass


def get_flasher_for_module(module, *args, **kwargs):
    """Returns the flasher class for the module type"""
    if module in RF100_MODULES:
        return Hcs08Flasher(*args, **kwargs)
    else:
        return ATMegaFlasher(*args, **kwargs)


def _get_file_path_for_version(metadata, module, version=None):
    img_dir = os.path.join(metadata._get_meta_dir(), '..', 'images')
    filename = metadata.get_metadata(module, version, aes=True)['filename']
    return os.path.join(img_dir, filename)


def firmware(profile, module, core, sniffer=False, override_file=None, timeout=180, progress_cb=None, finished_cb=None):
    """Flash firmware"""
    firmware = None
    if override_file:
        firmware = os.path.abspath(os.path.expanduser(override_file))
        log.warn("Overriding firmware path: %s", firmware)
    else:
        # Extend the path so we can find firmware support packages
        ensure_path_contains_python_lib()

        try:
            if sniffer:
                metadata_module_name = "snap_sniffer"
                metadata = importlib.import_module(metadata_module_name + '.metadata')
                log.debug("Looking for sniffer firmware for %s", metadata_module_name)

                firmware = _get_file_path_for_version(metadata, module, core)
            else:
                metadata_module_name = "snap_firmware_%s" % (core.replace('.', '_'),)
                metadata = importlib.import_module(metadata_module_name + '.metadata')
                log.debug("Looking for firmware for %s", metadata_module_name)

                firmware = _get_file_path_for_version(metadata, module, core)

            log.debug("Found firmware %s", firmware)
        except:
            log.exception("Couldn't find firmware module %s, pip install --extra-index-url https://update.synapse-wireless.com/pypi %s", metadata_module_name, metadata_module_name)
            raise

    # Convert SFI to HEX...
    if os.path.splitext(firmware)[1] == '.sfi':
        fd, path = tempfile.mkstemp(text=True)
        sfi_contents = open(firmware, 'rb').read()
        decompressText = bz2.decompress(sfi_contents)
        if decompressText is not None:
            tmpFile = os.fdopen(fd, 'w')
            tmpFile.write(decompressText)
            tmpFile.close()
            firmware = path

    cb = functools.partial(flasher, profile, module, firmware, "Flash Firmware", progress_cb=progress_cb, finished_cb=finished_cb)
    flash_result = None

    try:
        flash_result = IOLoop.current().run_sync(cb, timeout=timeout)
    except TimeoutError:
        log.error("Flash firmware timed out or aborted!")

    return flash_result


def default_nv(profile, module, timeout=30, progress_cb=None, finished_cb=None):
    """Factory default the NV params"""
    fd, path = tempfile.mkstemp(text=True)
    tempFile = os.fdopen(fd, 'w')

    if module in RF100_MODULES:
        tempFile.write(buildMagicSRec('N'))
    else:
        tempFile.write(buildMagicHRec('N'))

    tempFile.close()

    cb = functools.partial(flasher, profile, module, path, "Default NV Params", progress_cb=progress_cb, finished_cb=finished_cb)
    flash_result = None
    try:
        flash_result = IOLoop.current().run_sync(cb, timeout=timeout)
    except TimeoutError:
        log.error("Default NV Params timed out or aborted!")

    return flash_result


def erase_script(profile, module, timeout=30, progress_cb=None, finished_cb=None):
    """Erase the SNAPpy script"""
    fd, path = tempfile.mkstemp(text=True)
    tempFile = os.fdopen(fd, 'w')

    if module in RF100_MODULES:
        tempFile.write(buildMagicSRec('S'))
    else:
        tempFile.write(buildMagicHRec('S'))

    tempFile.close()

    cb = functools.partial(flasher, profile, module, path, "Erase SNAPpy Script", progress_cb=progress_cb, finished_cb=finished_cb)
    flash_result = None
    try:
        flash_result = IOLoop.current().run_sync(cb, timeout=timeout)
    except TimeoutError:
        log.error("Erase Script timed out or aborted!")

    return flash_result


@coroutine  # noqa
def flasher(profile, module, path, action, progress_cb=None, finished_cb=None):
    """Workhorse, actually runs the flasher"""
    result = Future()

    evScheduler = EventScheduler.EventScheduler()

    serialwrapper_path = os.path.dirname(serialwrapper.__file__)

    def on_finished():
        """Complete callback"""
        log.info("%s complete!", action)
        if finished_cb:
            finished_cb()
        result.set_result(True)

    sc_profile = profile.as_sc_params()

    f = get_flasher_for_module(module,
                               path,
                               scheduler=evScheduler,
                               finishedCallback=on_finished,
                               port=sc_profile.port,
                               type=sc_profile.type,
                               pathToUsbLibrary=serialwrapper_path)

    log.debug("Flasher Instance is: %r", f)
    log.debug("Flasher Vars: %r", vars(f))

    if module not in RF100_MODULES:
        # Force the flasher to idle
        f.state = f.STATE_IDLE

        def go_flasher():
            """Tell the flasher it can wait for incoming data now"""
            f.state = f.STATE_INCOMING_WAIT
    else:
        # RF100 / Hcs08Flasher doesn't have the same state machine
        def go_flasher():
            """Don't do anything"""
            pass

    def on_progress(counter):
        """Progress update callback"""
        log.debug("Got flasher update: %r", counter)
        if progress_cb:
            progress_cb(counter, f.max_progress)

    def on_error(error):
        """Error callback"""
        log.error("Flasher error: %r", error)
        result.set_exception(FlasherError(error))

    f.registerErrorCallback(on_error)
    f.registerProgressCallback(on_progress)

    log.warn("Restart your device. (We'll try to restart it for you if it's an SS200 / reset-on-DTR.)")
    if module in RF100_MODULES:
        log.warn("RF100s currently don't reset-on-DTR, you must reset them manually.")

    if os.path.exists('/etc/e20_version'):
        log.info("Looks like you're on an E20, we'll attempt to reset via GPIO 34")

        def e20_reset_on():
            """Toggle the E20 radio reset pin low"""
            try:
                with open('/sys/class/gpio/gpio34/value', 'w') as gpio:
                    gpio.write('0')
            except:
                log.exception("Couldn't reset SNAP node! (Do you need sudo?)")
                raise

        def e20_reset_off():
            """Toggle the E20 radio reset pin back to high"""
            try:
                with open('/sys/class/gpio/gpio34/value', 'w') as gpio:
                    gpio.write('1')
            except:
                log.exception("Couldn't reset SNAP node! (Do you need sudo?)")
                raise

        evScheduler.schedule(1.001, e20_reset_on)
        evScheduler.schedule(1.200, e20_reset_off)
        evScheduler.schedule(1.1, go_flasher)

    # If it's a SNAPStick, poke the reset line
    elif sc_profile.type == serialwrapper.PyserialDriver.PyserialWrapper.TYPE_SNAPSTICK:
        # Empirically determined schedule timing for reset...
        def bitbang_SS200_reset_on():
            """Bitbang the SS200 reset line low"""
            f.serialDrv.serial.setBitMode(0x10, 0x20)

        def bitbang_SS200_reset_off():
            """Bitbang the SS200 reset line back to high"""
            f.serialDrv.serial.setBitMode(0x11, 0x20)

        evScheduler.schedule(2.001, bitbang_SS200_reset_on)
        evScheduler.schedule(2.200, bitbang_SS200_reset_off)
        # Tell the flasher it can start looking at data after the resets
        evScheduler.schedule(2.100, go_flasher)

    else:
        def try_setDTR():
            """Try to set DTR"""
            try:
                log.debug("setting DTR")
                # Yep, this is inverted on purpose
                f.serialDrv.serial.setDTR(False)
            except:
                log.exception("Couldn't setDTR")

        def try_clearDTR():
            """Try to clear DTR"""
            try:
                log.debug("clearing DTR")
                # Yep, this is inverted on purpose
                f.serialDrv.serial.setDTR(True)
            except:
                log.exception("Couldn't setDTR")

        # Poke the DTR line
        evScheduler.schedule(1.001, try_setDTR)
        evScheduler.schedule(1.200, try_clearDTR)
        # Tell the flasher it can start looking at data after the resets
        evScheduler.schedule(1.100, go_flasher)

    PeriodicCallback(evScheduler.poll, 5).start()

    done = yield result
    raise Return(done)

################################
# These are copied from flasher, because the file they're in now also imports wx...
# Favor duplication over dealing with wxPython...


def s19_checksum(line):
    """Calculate the S19 checksum"""
    length = int(line[2:4], 16)
    pbytes = [int(line[i * 2:i * 2 + 2], 16) for i in range(1, length + 1)]
    return ~sum(pbytes) & 0xff


def niceHex(number):
    """Format a number in a nice hex way?"""
    numHex = hex(number)
    numHex = numHex[2:]  # chop off the leading '0x'
    if len(numHex) != 2:
        numHex = '0' + numHex  # tack on a leading '0' if not two digits

    numHex = numHex.upper()
    return numHex


def buildMagicSRec(cmdChar):
    """Build the magic S19 record used for defaulting NV params and erasing scripts"""
    magicAddr = '1080'
    magicKey = '524D44454D474B42'

    cmdHex = niceHex(ord(cmdChar))

    magicLen = niceHex((len(magicAddr) + len(magicKey) + len(cmdHex) + 2) / 2)  # +2 for trailing checksum, /2 to get bytes not chars

    srec = 'S1' + magicLen + magicAddr + magicKey + cmdHex

    checksum = s19_checksum(srec)
    srec = srec + niceHex(checksum)

    return srec


def buildMagicHRec(cmdChar, addr='02F0'):
    """Build the magic hex record used for defaulting NV params and erasing scripts"""
    magicKey = '524D44454D474B42'
    record = '09%s00%s%02X' % (addr, magicKey, ord(cmdChar))
    crc = (~sum(array.array('B', binascii.unhexlify(record))) + 1) % 2 ** 8

    # text = ':020000020000FC\n' # Header
    text = ':%s%02X\n' % (record, crc)
    text += ':00000001FF'  # End of file record

    return text
